# Copyright (C) 2019-2020  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import os

import pkg_resources


def get_sentry_release():
    main_package = os.environ.get("SWH_MAIN_PACKAGE")
    if main_package:
        version = pkg_resources.get_distribution(main_package).version
        return f"{main_package}@{version}"
    else:
        return None


def init_sentry(sentry_dsn, *, debug=None, integrations=[], extra_kwargs={}):
    if debug is None:
        debug = bool(os.environ.get("SWH_SENTRY_DEBUG"))
    sentry_dsn = sentry_dsn or os.environ.get("SWH_SENTRY_DSN")
    environment = os.environ.get("SWH_SENTRY_ENVIRONMENT")

    if sentry_dsn:
        import sentry_sdk

        sentry_sdk.init(
            release=get_sentry_release(),
            environment=environment,
            dsn=sentry_dsn,
            integrations=integrations,
            debug=debug,
            **extra_kwargs,
        )



# Copyright (C) 2019  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from os import path
from urllib.parse import unquote

import requests

from swh.core.pytest_plugin import requests_mock_datadir_factory


def test_get_response_cb_with_encoded_url(requests_mock_datadir):
    # The following urls (quoted, unquoted) will be resolved as the same file
    for encoded_url, expected_response in [
        ("https://forge.s.o/api/diffusion?attachments%5Buris%5D=1", "something"),
        (
            "https://www.reference.com/web?q=What+Is+an+Example+of+a+URL?&qo=contentPageRelatedSearch&o=600605&l=dir&sga=1",  # noqa
            "something else",
        ),
    ]:
        for url in [encoded_url, unquote(encoded_url)]:
            response = requests.get(url)
            assert response.ok
            assert response.json() == expected_response


def test_get_response_cb_with_visits_nominal(requests_mock_datadir_visits):
    response = requests.get("https://example.com/file.json")
    assert response.ok
    assert response.json() == {"hello": "you"}

    response = requests.get("http://example.com/something.json")
    assert response.ok
    assert response.json() == "something"

    response = requests.get("https://example.com/file.json")
    assert response.ok
    assert response.json() == {"hello": "world"}

    response = requests.get("https://example.com/file.json")
    assert not response.ok
    assert response.status_code == 404


def test_get_response_cb_with_visits(requests_mock_datadir_visits):
    response = requests.get("https://example.com/file.json")
    assert response.ok
    assert response.json() == {"hello": "you"}

    response = requests.get("https://example.com/other.json")
    assert response.ok
    assert response.json() == "foobar"

    response = requests.get("https://example.com/file.json")
    assert response.ok
    assert response.json() == {"hello": "world"}

    response = requests.get("https://example.com/other.json")
    assert not response.ok
    assert response.status_code == 404

    response = requests.get("https://example.com/file.json")
    assert not response.ok
    assert response.status_code == 404


def test_get_response_cb_no_visit(requests_mock_datadir):
    response = requests.get("https://example.com/file.json")
    assert response.ok
    assert response.json() == {"hello": "you"}

    response = requests.get("https://example.com/file.json")
    assert response.ok
    assert response.json() == {"hello": "you"}


def test_get_response_cb_query_params(requests_mock_datadir):
    response = requests.get("https://example.com/file.json?toto=42")
    assert not response.ok
    assert response.status_code == 404

    response = requests.get("https://example.com/file.json?name=doe&firstname=jane")
    assert response.ok
    assert response.json() == {"hello": "jane doe"}


requests_mock_datadir_ignore = requests_mock_datadir_factory(
    ignore_urls=["https://example.com/file.json"], has_multi_visit=False,
)


def test_get_response_cb_ignore_url(requests_mock_datadir_ignore):
    response = requests.get("https://example.com/file.json")
    assert not response.ok
    assert response.status_code == 404


requests_mock_datadir_ignore_and_visit = requests_mock_datadir_factory(
    ignore_urls=["https://example.com/file.json"], has_multi_visit=True,
)


def test_get_response_cb_ignore_url_with_visit(requests_mock_datadir_ignore_and_visit):
    response = requests.get("https://example.com/file.json")
    assert not response.ok
    assert response.status_code == 404

    response = requests.get("https://example.com/file.json")
    assert not response.ok
    assert response.status_code == 404


def test_data_dir(datadir):
    expected_datadir = path.join(path.abspath(path.dirname(__file__)), "data")
    assert datadir == expected_datadir



# Copyright (C) 2020  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from typing import TypeVar

from swh.core.api.classes import PagedResult as CorePagedResult
from swh.core.api.classes import stream_results

T = TypeVar("T")
TestPagedResult = CorePagedResult[T, bytes]


def test_stream_results_no_result():
    def paged_results(page_token) -> TestPagedResult:
        return TestPagedResult(results=[], next_page_token=None)

    # only 1 call, no pagination
    actual_data = stream_results(paged_results)
    assert list(actual_data) == []


def test_stream_results_no_pagination():
    input_data = [
        {"url": "something"},
        {"url": "something2"},
    ]

    def paged_results(page_token) -> TestPagedResult:
        return TestPagedResult(results=input_data, next_page_token=None)

    # only 1 call, no pagination
    actual_data = stream_results(paged_results)
    assert list(actual_data) == input_data


def test_stream_results_pagination():
    input_data = [
        {"url": "something"},
        {"url": "something2"},
    ]
    input_data2 = [
        {"url": "something3"},
    ]
    input_data3 = [
        {"url": "something4"},
    ]

    def page_results2(page_token=None) -> TestPagedResult:
        result_per_token = {
            None: TestPagedResult(results=input_data, next_page_token=b"two"),
            b"two": TestPagedResult(results=input_data2, next_page_token=b"three"),
            b"three": TestPagedResult(results=input_data3, next_page_token=None),
        }
        return result_per_token[page_token]

    # multiple calls to solve the pagination calls
    actual_data = stream_results(page_results2)
    assert list(actual_data) == input_data + input_data2 + input_data3
